package com.bull.cimero.pluginEditor.editors.model.BCModel;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * This class is the BindingComponent class. All the binding
 * components of the modelmust to extends this class.
 * It contain the functionnalities which permit to
 * change the MEP mode of this element
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class BindingComponentModel extends GeneriqueCimeroModel {

	/** constant for yes button in the msgbox. */
	public static final int YESBUTTON = 64;
	/** the max input for the router. */
	public static final int MAXINPUT = 5;
	/** The max output number for the router. */
	public static final int MAXOUTPUT = 5;
	/** This property will be draw in the property view. */
	public static final String INOUT = "MEP Mode";
	/** This variable is used to determine if the element model is in
	 * the IN mode MEP. */
	/*private*/protected boolean input;
	/** This variable is used to determine if the element model is in
	 * the out mode MEP. */
	/*private*/protected boolean output;
	//-- AZ(BEGIN)

	
	public static final String FLAG_INPUT_OUTPUT = "Flag Input Output";
//	-- AZ(END)
	
	/**  serial UID. */
	private static final long serialVersionUID = 1L;

	/**
	 * The constructor with parameters.
	 * @param name : name to set
	 */
	public BindingComponentModel(final String name) {
		super(name, 1, MAXINPUT);
		if (setInput(false)){
			setOutput(true);
		}
	}

	/**
	 * This function return the input value.
	 * @return boolean
	 */
	public final boolean isInput() {
		return input;
	}

	/**
	 * This function is used to set the input value.
	 * If there is connection that already exist and the user
	 * want to change the MEP mode, a message box appear
	 * and ask to the user if it want to delete all the connection
	 * before change the MEP mode. If it true, connection is delete.
	 * @param inputParam : the input boolean parameter
	 * @return boolean
	 */
	public /*final*/ boolean setInput(final boolean inputParam) {
		//if the new input == false and the old input == true,
		//we must to verify if the is connection with this element
		if ((this.input) && (!inputParam)) {
			//we verify if there is connection
			if (getIncomingConnections().size() != 0) {
				//we alert the user that there is connnection. And we propose it
				//to continu or stop.
				Shell monShell = new Shell();
				MessageBox msg = new MessageBox(monShell, SWT.YES | SWT.NO);
				msg.setMessage("Some connections are connected with "
						+ "this object !"
						+ " All connection will be delete"
						+ " ! Do you want to continue ?");
				msg.setText("Input connection exist");
				//the messagebox is open
				if (msg.open() == YESBUTTON) {
					//if the user had pressed YES
					this.input = inputParam;
					//we delete all the connections
					removeAllInput();
				} else {
					//if the user had pressed NO
					return false;
				}
			} else {
				this.input = inputParam;
			}
		} else {
			this.input = inputParam;
		}
		fireStructureChange(NAME, this);
		return true;
	}
	
	/**
	 * This function return the ouput value.
	 * @return boolean
	 */
	public final boolean isOutput() {
		return output;
	}

	/**
	 * This function is used to set the output value. If there is
	 * connection that already exist and the user want to change the
	 * MEP mode, a message box appear and ask to the user if
	 * it want to delete all the connection before change the MEP mode.
	 *  If it true, connection is delete.
	 * @param outputParam :
	 * @return boolean
	 */
	public /*final*/ boolean setOutput(final boolean outputParam) {
		//if the new input == false and the old input == true,
		//we must to verify if the is connection with this element
		if ((this.output) && (!outputParam)) {
			//we verify if there is connection
			if (getOutgoingConnections().size() != 0) {
				//we alert the user that there is connnection. And we propose it
				//to continu or stop.
				Shell monShell = new Shell();
				MessageBox msg = new MessageBox(monShell, SWT.YES | SWT.NO);
				msg.setMessage("Some connections are connected with"
						+ " this object "
						+ "! All connections will be delete ! Do you want "
						+ "to continue ?");
				msg.setText("Output connection exist");
				//the messagebox is open
				if (msg.open() == YESBUTTON) {
					//we delete all the connections
					removeAllOutput();
				} else {
					//if the user had pressed NO
					return false;
				}
			}
		}
		this.output = outputParam;
		fireStructureChange(NAME, this);
		return true;
	}

	

}
