package com.bull.cimero.pluginEditor.editors.model.SEModel;

import java.util.List;

import org.spagic.utils.SpagicLogger;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class People extends ServiceEngineModel {

	/** serialization number. */
	private static final long serialVersionUID = 1L;

	/** The xsl transformeur path. */
	private static final String XSLTPATH = "Xslt file Path";

	/** The name of the destination service property. */
	private static final String NAMEDEST = "destination service";

	private static final String INPUTMESSAGETYPE = "inputtype";

	private static final String OUTPUTMESSAGETYPE = "outputtype";

	/** The name of the mep mode property. */
	private static final String INOUTMEP = "in/out MEP";

	/** The default transformer name. */
	private static final String DEFAULTNAME = "myPeople";

	/** The size of the xslpath prefixe. */
//	private static final int XSLPREFIXESIZE = 5;

	/** the id of this element. */
	private static final String ID = "PEOPLE";

	/**
	 * The default constructor.
	 */
	public People() {
		super(DEFAULTNAME, 1, DEFAULTOUTIN);
		// add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(XSLTPATH, "file:");
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);

		setProperty(INPUTMESSAGETYPE, "");
		setProperty(OUTPUTMESSAGETYPE, "");
	}

	/**
	 * Path getter function.
	 * 
	 * @return String
	 */
	public final String getXsltPath() {
		return (String) getProperty(XSLTPATH);
	}

	/**
	 * path setter function.
	 * 
	 * @param pathParam :
	 *            the path of the xslt file
	 */
	public final void setXsltPath(final String pathParam) {
		setProperty(XSLTPATH, pathParam);
	}

	/**
	 * inoutMep getter function.
	 * 
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * 
	 * @param inOutMepParam :
	 *            in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class is
	 * valide or not.
	 * 
	 * @param showLog :
	 *            true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		// //name verification
		// if (getName().compareTo("") == 0) {
		// msg += "\n1 - Name => the transformer name cannot be null";
		// returnResult = false;
		// }
		// //xsl path verification
		// if (getXsltPath().compareTo("") == 0) {
		// msg += "\n2 - Xslt file path => set the xslt file path";
		// returnResult = false;
		// } else if (getXsltPath().substring(0 , XSLPREFIXESIZE)
		// .compareTo("file:") != 0) {
		// msg += "\n2 - Xslt file path => "
		// + " the path must to begin with \"file:\"";
		// returnResult = false;
		// } else {
		// //test if the xslt file exists
		// String subStr = getXsltPath().substring(XSLPREFIXESIZE ,
		// getXsltPath().length());
		// if (!FileUtils.getDefault().isFile((subStr))) {
		// msg += "\n2 - Xslt file path => file doesn't exist";
		// returnResult = false;
		// }
		// }
		// if (getNameFirstDestination().compareTo("") == 0) {
		// msg += "\n3 - Destination service => the destination of this "
		// + "element cannot be null!";
		// returnResult = false;
		// }
		// if (!returnResult) {
		// msg = msg.substring(1);
		// msg = "ELEMENT NAME : " + getName() + " \n" + msg;
		// if (showLog) {
		// CimeroLog.warningLog(msg);
		// }
		// }
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * 
	 * @param value :
	 *            the value of the property
	 * @param id :
	 *            the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			// we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			// retreive the list af all the objects name
			String[] lstStr = getLstObjectDrawWithInputConnection();
			// call the function to create a connection beetween
			// this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		} 
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * 
	 * @return Object
	 * @param id :
	 *            the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			// we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			// return the num of the name of the object
			String[] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		}
		else if (INPUTMESSAGETYPE.equals(id) || OUTPUTMESSAGETYPE.equals(id)) {
			Object ID = super.getProperty((String) id);
			if (ID instanceof String) {
				// First time component is used, the property is the empty string
				try {
					return Integer.parseInt((String)ID);
				} catch (Exception ex) {
					return 0;
				}
			}
			return ((Integer)ID).intValue();
		}
		else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * 
	 * @param id :
	 *            the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to create the
	 * descriptor
	 */
	protected final void createDescriptor() {
		int indice = 1;
		String[] lstStr = getLstObjectDrawWithInputConnection();
		String[] emptyOutputServices = new String[] {};
		
		String[] inputMapping = null;
		String[][] outputMapping = null;
		String[][] fileMapping = null;
		
		Ressources ressource = new Ressources();
		List < String > result = ressource.getStringNodeList(
					"/ressources/PeopleMessages.xml",
					"/messages/message");
		NodeList nodeList = ressource.getXPathNodeSet("/messages/message");
		
		inputMapping = new String[nodeList.getLength()];
		outputMapping = new String[nodeList.getLength()][];
		fileMapping = new String[nodeList.getLength()][];
		
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
						
			NamedNodeMap nodeMap = node.getAttributes();
			inputMapping[i] = nodeMap.getNamedItem("input").getNodeValue();
			
			NodeList childNodes = node.getChildNodes();
			
			// Count effective nodes
			int effectiveSize = 0;
			for (int j = 0; j < childNodes.getLength(); j++) {
				Node singleMapping = childNodes.item(j);
				if (singleMapping.getNodeName().equals("mapping"))
					effectiveSize++;
			}
			
			outputMapping[i] = new String[effectiveSize];
			fileMapping[i] = new String[effectiveSize];
			
			for (int j = 0, effectiveCounter = 0; j < childNodes.getLength(); j++) {
				
				Node singleMapping = childNodes.item(j);
				if (!singleMapping.getNodeName().equals("mapping"))
					continue;
				
				NamedNodeMap mappingAttributes = singleMapping.getAttributes();
				
				Node nodeOutput = mappingAttributes.getNamedItem("output");
				String outputValue = nodeOutput.getNodeValue();
				Node nodeFile = mappingAttributes.getNamedItem("file");
				String fileValue = nodeFile.getNodeValue();
				
				outputMapping[i][effectiveCounter] = outputValue;
				fileMapping[i][effectiveCounter] = fileValue;
				effectiveCounter++;
				
			}
			
		}
		
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(XSLTPATH, indice++ + " - select the xslt file path");

		addComboBoxDescriptor(INPUTMESSAGETYPE, indice++
				+ " - select the input message type", inputMapping);

		if (getPropertyValue(INPUTMESSAGETYPE) == null)
			addComboBoxDescriptor(OUTPUTMESSAGETYPE, indice++
					+ " - select the output message type", emptyOutputServices);
		else
			addComboBoxDescriptor(OUTPUTMESSAGETYPE, indice++
					+ " - select the output message type", outputMapping[((Integer)getPropertyValue(INPUTMESSAGETYPE)).intValue()]);

		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * 
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
