package com.bull.cimero.pluginEditor.editors.model.SEModel;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.commands.ConnectionCreateCommand;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Element;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.utils.propertyCombo.ComboBoxPropertyEditable;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The routeur model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RecipientList extends ServiceEngineModel {

	/** The number for the serialization. */
	private static final long serialVersionUID = 1L;
	/** the default value of input for the router. */
	public static final int DEFAUTLINDESTINATIONS = 5;
	/** The default value of output number for the router. */
	public static final int DEFAULTOUTDESTINATIONS = 5;
	/** The id for the number rule. */
	public static final String NBREGLE = "number of destination";
	/**The default routeur name.*/
	private static final String RECIPIENTLIST = "myRecipientList";
	/** This list contain all the rule of this routeur. */
	private List < Destination > maLstDestination = null;
	/**The id of this element.*/
	private static final String ID = "RECIPIENTLIST";

	/**.
	 * The default contructor
	 */
	public RecipientList() {
		super(RECIPIENTLIST, DEFAULTOUTDESTINATIONS, DEFAUTLINDESTINATIONS);
		//add name properties
		setProperty(NAME, RECIPIENTLIST);
		setProperty(NBREGLE, "1");
	}

	/**
	 * nbRules getter function.
	 * @return int
	 */
	public final int getNbDestinations() {
		return Integer.valueOf((String) getProperty(NBREGLE));
	}

	/**
	 * nbRules setters functions.
	 * @param numberRules : the numbers of rules
	 */
	public final void setNbDestinations(final int numberRules) {
		setProperty(NBREGLE, String.valueOf(numberRules));
	}

	/**
	 * The getter LstRules.
	 * @return List
	 */
	public final List < Destination > getMaLstDestination() {
		if (maLstDestination == null) {
			maLstDestination = new ArrayList < Destination >();
			addEltLst(new Destination(1));
		}
		return maLstDestination;
	}
	/**
	 * This function is used to add an Rule element in the list.
	 * @param param : the Rule element
	 */
	public final void addEltLst(final Destination param) {
		getMaLstDestination().add(param);
	}

	/**
	 * This function is used to remove the last rule
	 * of this routeur.
	 */
	public final void removeLastElement() {
		getMaLstDestination().remove(getMaLstDestination().size() - 1);
	}

	/**.
	 * This function is used by the validator. It is used to determine
	 * if this class is validate or not
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		boolean returnResult = true;
		String msg = "";
		if (getName().compareTo("") == 0) {
			msg += "\nThe name routeur cannot be null";
			returnResult = false;
		}
		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * set a property value.
	 * @param id : the id
	 * @param value : the value of this id
	 */
	public final void setPropertyValue(final Object id, Object value) {
		boolean find = false;
		if (id == NBREGLE) {
			//we compare the actual value with the last value
			int oldValue = getNbDestinations();
			int newValue = 0;
			//parse the Object in integer
			try {
				newValue =  Integer.valueOf((String) value);
				if ((newValue <= 0) || (newValue > 50)) {
					throw new Exception();
				}
				setNbOutputs(newValue + 1);
			} catch (Exception e) {
				SpagicLogger.error("The rule number must be a integer "
						+ "with a value between 0 and 50", e);
				newValue = oldValue;
				value = String.valueOf(newValue);
			}
			if (oldValue != newValue) {
				if (oldValue < newValue) {
					//we add new Rule
					for (int i = 0; i < newValue - oldValue; i++) {
						addEltLst(new Destination(oldValue + i + 1));
					}
				} else {
					//we remove the last Rule
					for (int i = 0; i < oldValue - newValue; i++) {
						removeLastElement();
					}
				}
			}
		} else {
			//we send this id and value to all rules
			for (int i = 0; i < getMaLstDestination().size(); i++) {
				if (getMaLstDestination().get(i).setPropertyValue(id, value)) {
					find = true;
					//tell the parts, property change --> refresh
					fireStructureChange(NAME, this);
				}
			}
		}
		if (!find) {
			super.setPropertyValue(id, value);
		}
	}

	/**
	 * get a property value.
	 * @param id : the id.
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (id.equals(NBREGLE)) {
			return (String) getProperty(NBREGLE);
		}
		//we send this object id to all rules
		for (int i = 0; i < getMaLstDestination().size(); i++) {
			Object result = getMaLstDestination().get(i).getPropertyValue(id);
			if (result != null) {
				return result;
			}
		}
		return super.getPropertyValue(id);
	}

	/**
	 * reset all property value.
	 * @param id : the id of the element
	 * @see org.eclipse.ui.views.properties.
	 * IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public final void resetPropertyValue(final Object id) {
		//do nothing
	}

	/**
	 * method to create descriptors.
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueModel#createDescriptor()
	 */
	protected final void createDescriptor() {

		//add name property
		IPropertyDescriptor descr;
		descr = new TextPropertyDescriptor(NAME, "name");
		((TextPropertyDescriptor) descr).setCategory("1 - RecipientList");
		addDescriptor(descr);

		descr = new TextPropertyDescriptor(NBREGLE, "number of recipients : ");
		((TextPropertyDescriptor) descr).setCategory("1 - RecipientList");
		addDescriptor(descr);

		//add rule descriptor
		int numRules = getMaLstDestination().size();
		for (int i = 0; i < numRules; i++) {
			IPropertyDescriptor[] ruleDescr = getMaLstDestination().get(i)
			.createPropertyDescriptor();
			for (int cptDescr = 0; cptDescr < ruleDescr.length; cptDescr++) {
				addDescriptor(ruleDescr[cptDescr]);
			}
		}
	}

	/**
	 * This function create a new connection between this object and
	 * the destination object. It return true if there is no problem
	 * during the connection creation.
	 * @param destination : the destination object
	 * @param obj : the rule that call this function
	 * @return boolean.
	 */
	public final boolean createNewConnection(
			final GeneriqueCimeroModel destination , final Destination obj) {
		try {
			ConnectionCreateCommand newConnect = new ConnectionCreateCommand();
			newConnect.setSource(this);
			newConnect.setTarget(destination);
			newConnect.execute();
			obj.setConn(newConnect.getConnection());
			return true;
		} catch (Exception e) {
			System.out.println(e);
			return false;
		}
	}

	/**
	 * This function returns the property of the previous element.
	 * @return a list of property name.
	 */
	public final String[] getLstPropPreviousElement() {
		//get the predessesor list
		List < GeneriqueCimeroModel > predElt = researchInput(this);
		//get the properties of all this element
		List < String > allResult = new ArrayList < String >();
		for (int i = 0; i < predElt.size(); i++) {
			GeneriqueCimeroModel elt = predElt.get(i);
			Ressources ressource = new Ressources();
			List < String > result = ressource.getStringNodeList(
						"/ressources/PropertyDefinition.xml",
						"/components/component[@id='" + elt.getComponentId()
						+ "']/properties");
			//we save this result
			for (int j = 0; j < result.size(); j++) {
				if (!allResult.contains(result.get(j))) {
					allResult.add(result.get(j));
				}
			}
		}

		String [] xpathResult = new String [allResult.size() + 1];
		xpathResult[0] = "";
		for (int i = 0; i < allResult.size(); i++) {
			xpathResult[i + 1] = allResult.get(i);
		}
		return xpathResult;
	}

	/**
	 * method to add an input.
	 * @see com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel
	 * #addInput(com.bull.cimero.pluginEditor.editors.model.Connection)
	 * @param connection : the connaction to add.
	 * @return true if the connection has been add.
	 */
	public final boolean addInput(final Connection connection) {
		boolean result = super.addInput(connection);
		//refresh the labels connection
		for (int i = 0; i < getMaLstDestination().size(); i++) {
			getMaLstDestination().get(i).myComboBox = null;
		}
		return result;
	}

	/**
	 * method to remove an input.
	 * @see com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel
	 * #removeInput(com.bull.cimero.pluginEditor.editors.model.Connection)
	 * @param connection : the connection to delete.
	 */
	public final void removeInput(final Connection connection) {
		super.removeInput(connection);
		//refresh the labels connection
		for (int i = 0; i < getMaLstDestination().size(); i++) {
			getMaLstDestination().get(i).myComboBox = null;
		}
	}

	/**
	 * This class contains all the property needed to create a routeur rule.
	 * @author Cedric MOUILLERON
	 * @author Pierre NOTEL
	 */
	public class Destination extends Element {

		/** list of available label. */
		private String [] lstLabel = null;
		/** editable combobox. */
		public transient ComboBoxPropertyEditable myComboBox = null;
		/** serial uid. */
		private static final long serialVersionUID = 1L;
		//CONSTANT
		
		/** default name. */
		private static final String DEFAULTNAME = "Destination ";
		
		/** id of destination service. */
		public static final String NAMEDEST = "destination service";

		/** The number of this rule. */
		private int numRule;
		/** The connection between this rule and the destination element. */
		private Connection conn = null;
		/** 3 constant. */
		public static final int THREE = 3;

		

		

		

		

		
		
		/**
		 * The Connection getter function.
		 * @return Connection
		 */
		public final Connection getConn() {
			return conn;
		}

		/**
		 * The connection setter function.
		 * @param connParam : the connection between this rule and the
		 *				destination object
		 */
		public final void setConn(final Connection connParam) {
			this.conn = connParam;
		}

		/**
		 * The default constructor.
		 * @param numRuleParam : the number of the rule
		 */
		public Destination(final int numRuleParam) {
			this.numRule = numRuleParam;
			
			//destination
			setProperty(numRule + NAMEDEST, "");
			

		}

		/**
		 * This function is used to calculate the size of the descriptor
		 * If this is a content comparaison, the size will be 3
		 * If this is a property comparison the size is 5.
		 * @return int
		 */
		public final int getSizeDescriptor() {
			int size = 1;
			
			return size;
		}

		/**
		 * This function is used to create the property descriptor of this
		 * Rule.
		 * @return IPropertyDescriptor[]
		 */
		public final IPropertyDescriptor[] createPropertyDescriptor() {
			//we set the size IpropertyDescriptor[]
			int tailleDescr = getSizeDescriptor();
			String prefixe = DEFAULTNAME + String.valueOf(this.numRule) + " : ";
			String indice = String.valueOf(this.numRule);

			int indiceTab = 0;
			PropertyDescriptor[] descr = new PropertyDescriptor[tailleDescr];
			//the content property : content or property
			descr[indiceTab] = new ComboBoxPropertyDescriptor(indice
					+ NAMEDEST,
					"select the destination",
					getLstObjectDrawWithInputConnection());
			descr[indiceTab++].setCategory(prefixe);
			return descr;
		}

		/**
		 *@param id : the id of the property
		 *@return Object
		 */
		public final Object getPropertyValue(final Object id) {
			String indice = String.valueOf(this.numRule);
			
			if ((indice + NAMEDEST).equals(id)) {
				String currentDestinationName = "";
				//we get the name of the connection
				if (getConn() != null) {
					//we test if the connection is always reference
					//in the OutgoindConnection
					if (isConnectionExistInOut(this.conn)) {
						//we get the name
						currentDestinationName =
							getConn().getTarget().getName();
					}
				}
				//return the num of the name  of the object
				String [] lstStr  = getLstObjectDrawWithInputConnection();
				for (int i = 0; i < lstStr.length; i++) {
					if (lstStr[i].compareTo(currentDestinationName) == 0) {
						return i;
					}
				}
			}
			return null;
		}

		/**
		 * This function is used to set the property value.
		 * @param id : the id of the property
		 * @param value : the new value of the property
		 * @return true if the property was finded.
		 */
		public final boolean setPropertyValue(final Object id,
				final Object value) {
			String indice = String.valueOf(this.numRule);
			
			if ((indice + NAMEDEST).equals(id)) {
				//we retreive the destination name
				int selecNum = Integer.parseInt(value.toString());
				//retreive the list af all the objects name
				String [] lstStr = getLstObjectDrawWithInputConnection();
				//call the function to create a connection beetween
				//this object and the selected object
				if (!refreshDestinationConnection(lstStr[selecNum])) {
					System.out.println("Error connection in routeur model");
				}
				return true;
			}
			return false;
		}

		/**
		 * we surcharge this function because the routeur model hasn't the same
		 * functionnement of the others SE model.
		 * @param nameDestination : the name of the destination
		 * @return boolean
		 */
		protected final boolean refreshDestinationConnection(
				final String nameDestination) {
			//if the Rule hasn't destination
			try {
				if (getConn() != null) {
					// we remove this connection
					getConn().getTarget().removeInput(getConn());
					removeOutput(getConn());
					setConn(null);
				}
				//we create a new connection
				GeneriqueCimeroModel tmp = getDiagram()
						.getGeneriqueModel(nameDestination);
				//if the generiquemodel exist
				if (tmp != null) {
					return createNewConnection(tmp, this);
				} else {
					//the user has selected ""
					return true;
				}
			} catch (Exception e) {
				return false;
			}
		}

		/**
		 * This function is used to determine if the current rule
		 * is valide. A rule is valide if all this properties are set.
		 * @return boolean
		 */
		private String isValidate() {
			String msg = "";
			String prefixe = "\n" + getName()
						+ " - " + DEFAULTNAME + String.valueOf(this.numRule)
						+ " : ";
	
			//test of the connection
			if ((getConn() == null)
					 || (!isConnectionExistInOut(this.conn))) {
				//no destination service
				msg += prefixe + "you must specify a destination "
				 		+ "for this rule !";
			}
			return msg;
		}

		
		
	}

	/**
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
