package com.bull.cimero.pluginEditor.utils.ressources;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.spagic.bpmn.conversion.Spagic1ToSpagic2Activator;
import org.spagic.utils.SpagicLogger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 * static class used to manipulate XML file.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class Ressources {

	/** The document. */
	private Document monDoc = null;

	/**
	 * convert string to inputStream.
	 * @param xml : xml string
	 * @return InputStream : wanted inputstream
	 */
	public static final InputStream parseStringToIS(final String xml) {
		String xmlStr = xml;
		if (xmlStr == null) {
			return null;
		}
		xmlStr = xmlStr.trim();
		java.io.InputStream in = null;
		try {
			in = new java.io.ByteArrayInputStream(xmlStr.getBytes("UTF-8"));
		} catch (Exception ex) {
			return null;
		}
		return in;
	}

	/**
	 * convert inputStream to String.
	 * @param is : inputstream
	 * @return a String
	 */
	public final String parseISToString(final java.io.InputStream is) {
		java.io.DataInputStream din = new java.io.DataInputStream(is);
		StringBuffer sb = new StringBuffer();
		try {
			String line = null;
			while ((line = din.readLine()) != null) {
				sb.append(line + "\n");
			}
		} catch (Exception ex) {
			ex.getMessage();
		} finally {
			try {
				is.close();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
		return sb.toString();
	}

	/**
	 * method used to test an XPath expression in a temp document.
	 * @param expression : XPath expression
	 * @return boolean : true if XPAth expression is valid
	 */
	public static final boolean testXPath(final String expression) {
		Document documentTmp = null;
		try {
			//init of the document builder
			DocumentBuilderFactory fabrique =
				DocumentBuilderFactory.newInstance();
			//creation of the document constructor
			DocumentBuilder constructeur = fabrique.newDocumentBuilder();
			//reading the ressources file
			String tt = "<test></test>";
			InputStream bibi = parseStringToIS(tt);
			documentTmp = constructeur.parse(bibi);
		} catch (Exception e) {
			//can't test --> so return valid
			return true;
		}
		try {
			//XPath creation
			XPathFactory fabrique = XPathFactory.newInstance();
			XPath xpath = fabrique.newXPath();
			//XPath evaluation
			XPathExpression exp = xpath.compile(expression);
			//we want the coresponding node
			String str = (String) exp.evaluate(documentTmp,
					XPathConstants.STRING);
		} catch (XPathExpressionException xpee) {
			return false;
		}
		//no exception --> valid expression
		return true;
	}


	/**
	 * method used to get a nodeset associated to a
	 * XPath expression in the document.
	 * @param expression : XPath expression
	 * @return NodeSet : wanted nodeset
	 */
	public final NodeList getXPathNodeSet(final String expression) {
		NodeList nodeList = null;
		try {
			//XPath creation
			XPathFactory fabrique = XPathFactory.newInstance();
			XPath xpath = fabrique.newXPath();
			//XPath evaluation
			XPathExpression exp = xpath.compile(expression);
			//we want the coresponding node
			nodeList = (NodeList)
			exp.evaluate(getDocument(), XPathConstants.NODESET);
		} catch (XPathExpressionException xpee) {
			return null;
		}
		return nodeList;
	}

	/**
	 * method used to get a node associated to a
	 * XPath expression in the document.
	 * @param expression : XPath expression
	 * @return Node : wanted node
	 */
	public final Node getXPathNode(final String expression) {
		Node node = null;
		try {
			//XPath creation
			XPathFactory fabrique = XPathFactory.newInstance();
			XPath xpath = fabrique.newXPath();
			//XPath evaluation
			XPathExpression exp = xpath.compile(expression);
			//we want the coresponding node
			node = (Node) exp.evaluate(getDocument(), XPathConstants.NODE);
		} catch (XPathExpressionException xpee) {
			return null;
		}
		return node;
	}

	/**
	 * method used to get a node value associated to a
	 * XPath expression in the document.
	 * @param expression : XPath expression
	 * @return String : wanted node value
	 */
	public final String getXPathValue(final String expression) {
		String str = null;
		try {
			//XPath creation
			XPathFactory fabrique = XPathFactory.newInstance();
			XPath xpath = fabrique.newXPath();
			//XPath evaluation
			XPathExpression exp = xpath.compile(expression);
			//we want the coresponding node
			str = (String)
			exp.evaluate(getDocument(), XPathConstants.STRING);
		} catch (XPathExpressionException xpee) {
			return null;
		}
		return str;
	}

	/**
	 * get the current document.
	 * @return the monDoc.
	 */
	public final Document getDocument() {
		return monDoc;
	}

	/**
	 * set the document with an url file.
	 * @param monFic : the xml urlfile.
	 */
	public final void setDocument(final String monFic) {
		try {
			//init of the document builder
			DocumentBuilderFactory fabrique =
				DocumentBuilderFactory.newInstance();
			//creation of the document constructor
			DocumentBuilder constructeur = fabrique.newDocumentBuilder();
			this.monDoc = constructeur.parse(monFic);
		} catch (ParserConfigurationException e) {
			SpagicLogger.error("", e);
		} catch (SAXException e) {
			SpagicLogger.error("", e);
		} catch (IOException e) {
			SpagicLogger.error("", e);
		}
	}

	/**
	 * set the document with an input string.
	 * @param in : the input string.
	 */
	public final void setDocument(final InputStream in) {
		try {
			//init of the document builder
			DocumentBuilderFactory fabrique =
				DocumentBuilderFactory.newInstance();
			//creation of the document constructor
			DocumentBuilder constructeur = fabrique.newDocumentBuilder();
			this.monDoc = constructeur.parse(in);
		} catch (ParserConfigurationException e) {
			SpagicLogger.error("", e);
		} catch (SAXException e) {
			SpagicLogger.error("", e);
		} catch (IOException e) {
			SpagicLogger.error("", e);
		}
	}

	/**
	 * return the list of String that return with this xpath rule in
	 * the specify file.
	 * @param fileName : the name of the xml file
	 * @param xPathrule : the xpath rule.
	 * @return the xpath result in a string table.
	 */
	public final List < String > getStringNodeList(final String fileName,
			final String xPathrule) {
		//init the return string
		String [] lstString = null;
		String fullPath = Spagic1ToSpagic2Activator.getRessourcePath(fileName);
		if (fullPath == null) {
			return new ArrayList < String >();
		} else if (fullPath.substring(
							0, "file:".length()).compareTo("file:") == 0) {
			fullPath = fullPath.substring("file:".length());
		}
		setDocument(fullPath);
		//run the xpath rule
		String result = getXPathValue(xPathrule);
		//remove all the space and tabulation
		result = result.replace('\t', ' ');
		result = result.replace('\n', ' ');
		//convert the list into string table
		lstString = result.split(" ");
		//remove all the space element
		List < String > lstRes = new ArrayList < String >();
		for (int i = 0; i < lstString.length; i++) {
			String maStr = lstString[i];
			if (!maStr.equals("")) {
				lstRes.add(maStr);
			}
		}
		return lstRes;
	}
}
