package it.eng.components.cimero;


import java.util.StringTokenizer;

import org.spagic.bpmn.conversion.Spagic1ToSpagic2Activator;
import org.spagic.utils.SpagicLogger;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The pipeline model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 */
/*
 * Configuration example from CustomComponents
	<COMPONENT 
		id="WS_Anagrafe-ricerca_soggetto"
		paletteName="WS_Anagrafe-ricerca_soggetto" 
		description="WS_Anagrafe-ricerca_soggetto"
		paletteImage="icons/ico_talend.jpg" 
		editorImage="icons/talendetl.gif"
		errorImage="icons/talendetl.gif"
		class="it.eng.components.cimero.GenericEnricher"
		locationURI="http://srv-cedaf01/ws_anagrafe/ws_anagrafe.asmx"
		soap="true"
		soapVersion="1.1"
		soapAction="http://www.cedaf.it/schema/wm_parametri"
		soapActionCombo="http://www.cedaf.it/schema/wm_soggetto,
		http://www.cedaf.it/schema/wm_famiglia,
		http://www.cedaf.it/schema/wm_immagine,
		http://www.cedaf.it/schema/wm_ricerca_soggetto,
		http://www.cedaf.it/schema/wm_attiva_procedimento,
		http://www.cedaf.it/schema/wm_lista_procedimenti,
		http://www.cedaf.it/schema/wm_procedimento,
		http://www.cedaf.it/schema/wm_parametri,
		http://www.cedaf.it/schema/wm_stato_soggetto"
		factoryClass="it.eng.components.cimero.GenericEnricherFactory"
		category="Anagrafe"
		/>
 */

public class GenericEnricher extends ServiceEngineModel {

	/** Serial UID. */
	private static final long serialVersionUID = 1L;
	/** Default name. */
	private static final String DEFAULTNAME = "myGenericEnricher";
	/**the id of this element.*/
	private static final String ID = "ENRICHER";
	/**The location uri of the WS transformeur path.*/
	private static final String LOCATIONURI = "Location URI";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The operation to call on the WS property.*/
	private static final String SOAPREQUEST = "Soap request";
	private static final String SOAP_ACTION = "SOAP_ACTION";
	/**
	 * store the value of the soapActionCombo
	 */
	private static final String SOAP_ACTION_FROM_COMBO = "SOAP_ACTION_FROM_COMBO";
	private String soapActionCombo[];
	private static final String SOAP_VERSION = "SOAP_VERSION";
	private static final String[] SOAP_VERSION_COMBO = {"1.2", "1.1"};

	/**
	 * default constructor.
	 */
	public GenericEnricher() {
		super(DEFAULTNAME, 1, DEFAULTOUTIN);
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(LOCATIONURI, "");
		setProperty(NAMEDEST, "");
		setProperty(SOAPREQUEST, 0);
		setProperty(SOAP_ACTION, "");
		setProperty(SOAP_VERSION, "1.2");
		setStandard(true);
	}
	
	public void configureCustomComponent(){
		
		Ressources ressource = new Ressources();
		String fullPath = Spagic1ToSpagic2Activator.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT");
			Node node = null;
			NamedNodeMap attributes = null;
			String tmpId  = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				tmpId = attributes.getNamedItem("id").getNodeValue();
				if (tmpId.equalsIgnoreCase(getCustomComponentId())){
					setLocationURI(attributes.getNamedItem("locationURI").getNodeValue());
					boolean soaprequest = Boolean.valueOf(attributes.getNamedItem("soap").getNodeValue());
					if (soaprequest){
						setProperty(SOAPREQUEST, 0);
					}else{
						setProperty(SOAPREQUEST, 1);
					}
					
					String customSoapVersion = attributes.getNamedItem("soapVersion").getNodeValue();
					setSoapVersionFromString(customSoapVersion);
					Node soapActionNode = attributes.getNamedItem("soapAction");
					setSoapAction(soapActionNode != null? soapActionNode.getNodeValue() : "");

					Node soapActionComboItem = attributes.getNamedItem("soapActionCombo");
					StringTokenizer soapActionTkz = new StringTokenizer(
 						soapActionComboItem!=null? soapActionComboItem.getNodeValue() : "", ",");
					if (soapActionTkz.hasMoreTokens()) {
						soapActionCombo = new String[soapActionTkz.countTokens()];
						for (int idx = 0; soapActionTkz.hasMoreTokens(); idx++) {
							soapActionCombo[idx] = soapActionTkz.nextToken().trim();
						}
						setProperty(SOAP_ACTION_FROM_COMBO, getSoapActionComboIdx(getSoapAction()));
					}
				}
			}
		}
	}

	/**
	 * get the location URI.
	 * @return Returns the locationURI.
	 */
	public final String getLocationURI() {
		return (String) getProperty(LOCATIONURI);
	}

	/**
	 * set the location URI.
	 * @param locationURIParam The locationURI to set.
	 */
	public final void setLocationURI(final String locationURIParam) {
		setProperty(LOCATIONURI, locationURIParam);
	}

	/**
	 * get the soapRequest.
	 * @return the soapRequest
	 */
	public final boolean isSoapRequest() {
		return ((Integer) getProperty(SOAPREQUEST) == 0);
	}

	/**
	 * set the soapRequest.
	 * @param soapRequestP the soapRequest to set
	 */
	public final void setSoapRequest(final boolean soapRequestP) {
		setProperty(SOAPREQUEST, soapRequestP);
	}
	
	public final String getSoapActionFromCombo() {
		Integer intValue = (Integer)getProperty(SOAP_ACTION_FROM_COMBO);
		if (intValue == null) { return ""; }
		return soapActionCombo[intValue];
	}
	
	public final Integer getSoapActionComboIdx(String action) {
		for (int i = 0; soapActionCombo != null && i < soapActionCombo.length; i++) {
			if (soapActionCombo[i].equals(action)) { return i; }
		}
		return soapActionCombo != null && soapActionCombo.length > 0? 
			0 : null;
	}
	
	public final String getSoapAction() {
		Object objValue = getProperty(SOAP_ACTION);
		return (objValue != null)? (String)objValue : "";
	}
	
	public final void setSoapAction(String action) {
		setProperty(SOAP_ACTION, action);
	}
	
	public final String getSoapVersion(){
		Integer intType = (Integer)getProperty(SOAP_VERSION);
		if (intType != null) {
			return SOAP_VERSION_COMBO[intType];
		}
		return SOAP_VERSION_COMBO[0];
	}
	
	public final void setSoapVersionFromString(String version) {
		Integer versionValue = null;
		for (int i = 0; i < SOAP_VERSION_COMBO.length; i++) {
			if (SOAP_VERSION_COMBO[i].equals(version)) {
				versionValue = i;
			}
		}
		setProperty(SOAP_VERSION, versionValue != null? versionValue : 0);
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	@Override
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the enricher name cannot be null";
			returnResult = false;
		}
		//location URI verification
		if (getLocationURI().compareTo("") == 0) {
			msg += "\n2 - location URI => set the location URI";
			returnResult = false;
		}
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n4 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	@Override
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name");
		//addTextDescriptor(LOCATIONURI, indice++
		//		+ " - select the location URI of the WS");
		//addComboBoxDescriptor(SOAPREQUEST, indice++ + " - soap request ? :",
		//				new String[]{"true", "false"});
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
		if (soapActionCombo != null && soapActionCombo.length > 0) {
			addComboBoxDescriptor(SOAP_ACTION_FROM_COMBO, indice++ 
				+" - select soap action", soapActionCombo);
		}
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	@Override
	public final String getComponentId() {
		return ID;
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (id == SOAP_VERSION) {
			Object soapVersion = getProperty(SOAP_VERSION);
			return (soapVersion != null &&(soapVersion instanceof Integer))? soapVersion : 0;
		} else if (id == SOAP_ACTION) {
			return getSoapAction();
		} else {
			return super.getPropertyValue(id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public final void resetPropertyValue(final Object id) {
	}

}
