package it.eng.components.cimero;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;


 /**
 * The Generic JDBC model class.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class GenericTalendComponent extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The location uri of the WS transformeur path.*/
	//private static final String LOCATIONURI = "Location URI";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myTalend";
	
	private static final String TALEND_JOB_CLASS_NAME = "talendJobClassName";
	private static final String TALEND_JOB_CLASS_DESC = "talendJobClassDescription";
	private static final String TALEND_JOB_ZIP_PATH = "talendJobZipPath";
	private List inputParameters = null;
	/**the id of this element.*/
	private static final String ID = "TALENDCOMPONENT";
	
	
	/**
	 * The default constructor.
	 */
	public GenericTalendComponent() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
		setProperty(TALEND_JOB_ZIP_PATH, "");
		setProperty(TALEND_JOB_CLASS_NAME, "");
		setProperty(TALEND_JOB_CLASS_DESC, "");
	}
	
	public void configureCustomComponent(){
	}
	
	public String getJobClassName(){
		return (String) getProperty(TALEND_JOB_CLASS_NAME);
	}
	
	public String getJobClassDescription(){
		return (String) getProperty(TALEND_JOB_CLASS_DESC);
	}
	
	public String getJobZipPath(){
		return (String) getProperty(TALEND_JOB_ZIP_PATH);
	}
	
	public List getInputParameters() {
		return inputParameters;
	}

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		return true;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		if (id == TALEND_JOB_ZIP_PATH) {
			if(!((String)super.getPropertyValue(id)).equals(value)){ 
				String path = value.toString();
				Properties properties = new Properties();
				inputParameters = null;
				try {
					ZipFile talendZip = new ZipFile(path);
					ZipEntry spagicFile = talendZip.getEntry("spagic.properties");
					properties.load(talendZip.getInputStream(spagicFile));
					//retrieve input/output parameters
					Enumeration keyParameters = properties.propertyNames();
					String nameParameter= "";
					String valueParameter = "";
					
					while(keyParameters.hasMoreElements()) {
						nameParameter = (String)keyParameters.nextElement();
						valueParameter = (String)properties.getProperty(nameParameter);
						super.setProperty(nameParameter, valueParameter);
						if(!nameParameter.equals(TALEND_JOB_CLASS_NAME) && !nameParameter.equals(TALEND_JOB_CLASS_DESC)){
							if (inputParameters == null)
								inputParameters = new ArrayList();
							inputParameters.add(nameParameter);
						}
					}
					
				}catch(ZipException ze) {
					SpagicLogger.error("problem when created parameter", ze);
				}catch(IOException ioe) {
					SpagicLogger.error("problem when created parameter", ioe);
				}
			}
		}
		
		super.setPropertyValue(id, value);
	}
	
//	private final void removeParametersProperties() {
//		Set keysProperties = getPropertyKeys();
//		Iterator itKey = keysProperties.iterator();
//		String keyProperty;
//		while (itKey.hasNext()) {
//			keyProperty = (String)itKey.next();
//			if(keyProperty.startsWith(PREF_INPUT_PARAMETERS)
//				|| keyProperty.startsWith(PREF_OUTPUT_PARAMETERS)) {
//				super.setProperty(keyProperty, null);
//				fireStructureChange(NAME, this);
//			}	
//		}
//	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		}else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	@Override
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(TALEND_JOB_ZIP_PATH, indice++ + " - Talend Job Zip Path");
		addTextDescriptor(TALEND_JOB_CLASS_NAME, indice++ + " - Talend Job Full Class Name");
		addTextDescriptor(TALEND_JOB_CLASS_DESC, indice++ + " - Talend Job Class Description");
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
		
		//create text descriptor for inpur output parameters
		String categoryInputParameters = "INPUT PARAMETERS";
		int index;
		if (inputParameters != null)
			for (index=0; index<inputParameters.size(); index++)
				addTextDescriptor((String)inputParameters.get(index), (String)inputParameters.get(index), categoryInputParameters);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	
}
