/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class PropertyDialog extends TitleAreaDialog {

	private Composite aComposite = null;
	private CLabel propertyKeyLabel = null;
	private Text propertyKeyText = null;

	private CLabel propertyValueLabel = null;
	private Text propertyValueTextField = null;
	private Property property = null;
	
	public PropertyDialog(Shell parentShell, Property property) {
		super(parentShell);
		this.property = property;
	}
	/**
	 * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	 *      a title and a message
	 */
	public void create() {
		super.create();
		if (property.getKey() != null) {
			setTitle("Update Property");
		}else{
			setTitle("New Property");
		}

	}

	protected Control createDialogArea(Composite parent) {

		aComposite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout();
		layout.marginWidth = 15;
		layout.marginHeight = 10;
		layout.numColumns = 5;
		aComposite.setLayout(layout);
		aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;

		// Riga 2
		propertyKeyLabel = new CLabel(aComposite, SWT.RIGHT);
		propertyKeyLabel.setText(" Key ");
		propertyKeyText = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		propertyKeyText.setLayoutData(data);
		
		
		// Riga 2
		propertyValueLabel = new CLabel(aComposite, SWT.RIGHT);
		propertyValueLabel.setText(" Value ");
		propertyValueTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		propertyValueTextField.setLayoutData(data);
		
		if (this.property.getKey() != null){
			propertyKeyText.setText(property.getKey());
			propertyValueTextField.setText(property.getValue());
			propertyKeyText.setEditable(false);
		}else{
			propertyKeyText.setText("");
			propertyValueTextField.setText("");
			propertyKeyText.setEditable(true);
		}

		return parent;
	}

	protected void okPressed() {
		if (property.getKey() != null)
			modifyProperty();
		else
			createProperty();
	}
	
	protected void modifyProperty() {
		if (propertyValueTextField.getText().trim().length() > 0) {
			property.setValue(propertyValueTextField.getText());
			close();
		} else {
			MessageDialog
					.openError(getShell(), "Spagic",
							"To modify a property you must set the the value");
		}
	}

	protected void createProperty() {
		if (propertyKeyText.getText().trim().length() > 0
				&& propertyValueTextField.getText().trim().length() > 0) {
			
			property.setKey(propertyKeyText.getText());
			property.setValue(propertyValueTextField.getText());

			close();
		} else {
			MessageDialog
					.openError(getShell(), "Spagic",
							"To add a property you must set the key and the value");
		}
	}

}
